'use strict';

exports.__esModule = true;
exports.default = Composer;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Composer({
  components,
  children,
  renderPropName,
  mapResult
}) {
  if (typeof children !== 'function') {
    return null;
  }

  /**
   * Recursively build up elements from props.components and accumulate `results` along the way.
   * @param {Array.<ReactElement|Function>} components
   * @param {Array} results
   * @returns {ReactElement}
   */
  function chainComponents(components, results) {
    // Once components is exhausted, we can render out the results array.
    if (!components[0]) {
      return children(results);
    }

    return _react2.default.cloneElement(
    // Each props.components entry is either an element or function [element factory]
    // When it is a function, produce an element by invoking it with currently accumulated results.
    typeof components[0] === 'function' ? components[0](results) : components[0],
    // Enhance the element's props with the render prop.
    {
      [renderPropName]() {
        return chainComponents(
        // Remove the current component and continue.
        components.slice(1),
        // results.concat([mapped]) ensures [...results, mapped] instead of [...results, ...mapped]
        results.concat(mapResult ? [mapResult.apply(null, arguments)] : arguments[0]));
      }
    });
  }

  return chainComponents(components, []);
}

Composer.propTypes = {
  children: _propTypes2.default.func,
  components: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.element, _propTypes2.default.func])),
  renderPropName: _propTypes2.default.string,
  mapResult: _propTypes2.default.func
};

Composer.defaultProps = {
  components: [],
  renderPropName: 'children'
};